@@ -2,11 +2,12 @@ import json
 import queue
 import re
 import os
+import time
 import traceback
 from pathlib import Path
 
 from netmiko import ConnectHandler
-from netmiko import NetmikoAuthenticationException
+from netmiko import NetmikoAuthenticationException, SSHDetect
 from netmiko.ssh_autodetect import SSHDetect
 import requests
 import yaml
@@ -46,7 +47,7 @@ class dyagram:
         self.changes_in_state = None
         self.site = self.get_current_site()
         self.state_exists = self.does_state_exist()
-        self._neighbor_template = {
+        self.lldp_neighbor_template = {
             "hostname": "",
             "local_port": "",
             "neighbor_port": "",
@@ -55,6 +56,12 @@ class dyagram:
 
         self._load_creds()
         self._load_inventory()
+        self.netmiko_args_template = {"device_type": "",
+                                      "host": "",
+                                      "username": self.username,
+                                      "password": self.password,
+                                      "secret": self.password,
+                                      "banner_timeout": 200}
 
 
 
@@ -73,7 +80,6 @@ class dyagram:
 
             file = open(r".info/info.json", 'r')
             info = json.load(file)
-            #print(info['current_site'])
             file.close()
             return info['current_site']
         except:
@@ -113,13 +119,16 @@ class dyagram:
             self.pbar.update(self.pbar_update_int)
             self.log.info(f"DEVICE: {device} - RESTCONF : Discovering LLDP Neighbors - SUCCESSFUL")
         except:
+            try:
             # try ssh
-            self.log.info(f"DEVICE: {device} - RESTCONF : Discovering LLDP Neighbors - FAILURE")
-            self.log.info(f"DEVICE: {device} - SSH : Discovering LLDP Neighbors - START")
-            self._discover_lldp_neighbors_by_ssh(device)
-            self.pbar.update(self.pbar_update_int)
-            self.log.info(f"DEVICE: {device} - SSH : Discovering LLDP Neighbors - SUCCESSFUL")
-
+                self.log.info(f"DEVICE: {device} - RESTCONF : Discovering LLDP Neighbors - FAILURE")
+                self.log.info(f"DEVICE: {device} - SSH : Discovering LLDP Neighbors - START")
+                self._discover_lldp_neighbors_by_ssh(device)
+                self.pbar.update(self.pbar_update_int)
+                self.log.info(f"DEVICE: {device} - SSH : Discovering LLDP Neighbors - SUCCESSFUL")
+            except Exception:
+                tb = self.get_traceback()
+                self.log.info(f"DEVICE: {device} EXCEPTION01 THROWN IN __discover_lldp_neighbors : {tb}")
 
     def discover(self):
 
@@ -129,53 +138,62 @@ class dyagram:
 
         :return:
         """
-        print(f'Discovering site "{self.site}"')
-        self.pbar = tqdm(total=100,
-                         bar_format=Fore.LIGHTBLUE_EX + "{l_bar}{bar:20}|") if not self.verbose else tqdm(total=100,
-                                                                                                     bar_format="{l_bar}{bar}|",
-                                                                                                     disable=True)
+        try:
+            print(f'Discovering site "{self.site}"')
+            self.pbar = tqdm(total=100,
+                             bar_format=Fore.LIGHTBLUE_EX + "{l_bar}{bar:20}|") if not self.verbose else tqdm(total=100,
+                                                                                                         bar_format="{l_bar}{bar}|",
+                                                                                                         disable=True)
 
-        executor = ThreadPoolExecutor(max_workers=30)
+            executor = ThreadPoolExecutor(max_workers=30)
 
-        queue_empty = False
-        while not queue_empty:
+            queue_empty = False
+            while not queue_empty:
+                try:
+                    device = self._devices_to_query.get_nowait()
+                    self.log.info(f"Pulled device {device} from Queue")
+                    self.topology['devices'].append({'hostname': "", 'inventory_ip': device, 'layer2': {}, 'routes': [], 'dynamic_routing_neighbors': {}})
+
+                    executor.submit(self.__discover_lldp_neighbors, device)
+                    self.log.info(f"DEVICE: {device} - Submitted for discover_lldp_neighbors")
+                    executor.submit(self.discover_routes, device)
+                    self.log.info(f"DEVICE: {device} - Submitted for discover_routes")
+                    executor.submit(self.discover_dynamic_routing_neighbors, device)
+                    self.log.info(f"DEVICE: {device} - Submitted for discover_dynamic_routing_neighbors")
+
+                    self._devices_queried.append(device)
+
+                except queue.Empty:
+                    self.log.info(f"QUEUE EMPTY")
+                    queue_empty = True
+                except Exception:
+                    tb = self.get_traceback()
+                    self.log.info(f"EXCEPTION THROWN IN PULLING FROM QUEUE: {tb}")
+
+
+            executor.shutdown(wait=True)
+
+            #sort topology by hostname to compare to previous
+
+            self.topology['devices'] = sorted(self.topology['devices'], key=lambda d: d['hostname'])
+
+            if not self.state_exists:
+                self.export_state()
+            else:
+                self.compare_states()
+
+            self.pbar.close()
+            if self.changes_in_state:
+                print("\nChanges in state!")
+            elif not self.changes_in_state and self.state_exists:
+                print("\nNo changes in state")
+        except:
             try:
-                device = self._devices_to_query.get_nowait()
-                self.log.info(f"Pulled device {device} from Queue")
-                self.topology['devices'].append({'hostname': "", 'inventory_ip': device, 'layer2': {}, 'routes': []})
-
-                executor.submit(self.__discover_lldp_neighbors, device)
-                self.log.info(f"DEVICE: {device} - Submitted for discover_lldp_neighbors")
-                executor.submit(self.discover_routes, device)
-                self.log.info(f"DEVICE: {device} - Submitted for discover_routes")
-                #executor.submit(self.__discover_dynamic_routing_neighbors, device)
-
-                self._devices_queried.append(device)
-
-            except queue.Empty:
-                self.log.info(f"QUEUE EMPTY")
-                queue_empty = True
-            except Exception:
                 tb = self.get_traceback()
-                self.log.info(f"EXCEPTION THROWN IN PULLING FROM QUEUE: {tb}")
-
-
-        executor.shutdown(wait=True)
-
-        #sort topology by hostname to compare to previous
-
-        self.topology['devices'] = sorted(self.topology['devices'], key=lambda d: d['hostname'])
-
-        if not self.state_exists:
-            self.export_state()
-        else:
-            self.compare_states()
-
-        self.pbar.close()
-        if self.changes_in_state:
-            print("\nChanges in state!")
-        elif not self.changes_in_state and self.state_exists:
-            print("\nNo changes in state")
+                self.log.info(f"DEVICE: {device} MAIN EXCEPTION : {tb}")
+            except:
+                tb = self.get_traceback()
+                self.log.info(f"DEVICE: UNKNOWN - MAIN EXCEPTION : {tb}")
 
 
     def does_state_exist(self):
@@ -188,6 +206,7 @@ class dyagram:
         json.dump(self.topology, file)
         file.close()
 
+
     def compare_states(self):
 
         file = open(rf"{self.site}/state.json", 'r')
@@ -211,58 +230,78 @@ class dyagram:
             self.pbar.update(self.pbar_update_int)
             self.log.info(f"DEVICE: {device} - RESTCONF : Discovering routes - SUCCESSFUL")
         except:
-            self.log.info(f"DEVICE: {device} - RESTCONF : Discovering routes - FAILURE")
-            self.log.info(f"DEVICE: {device} - SSH : Discovering routes - START")
-            routes = self.discover_routes_ssh(device)
-            self.pbar.update(self.pbar_update_int)
-            self.log.info(f"DEVICE: {device} - SSH : Discovering routes - SUCCESSFUL")
+            try:
+                self.log.info(f"DEVICE: {device} - RESTCONF : Discovering routes - FAILURE")
+                self.log.info(f"DEVICE: {device} - SSH : Discovering routes - START")
+                self.discover_routes_ssh(device)
+                self.pbar.update(self.pbar_update_int)
+                self.log.info(f"DEVICE: {device} - SSH : Discovering routes - SUCCESSFUL")
+            except:
+                tb = self.get_traceback()
+                self.log.info(f"DEVICE: {device} EXCEPTION01 THROWN IN discover_routes : {tb}")
 
         return True
 
-    def discover_routes_restconf(self):
+    def discover_routes_restconf(self, device):
         return None
 
     def discover_routes_ssh(self, device):
 
-        CONN_SET = False
+        # CONN_SET = False
 
-        autodetect_netmiko_args = {"device_type": "autodetect",
-                                   "host": device,
-                                   "username": self.username,
-                                   "password": self.password}
 
-        netmiko_args = {"device_type": "",
+        os = self.get_device_type(device)
+
+        netmiko_args = {"device_type": os,
                         "host": device,
                         "username": self.username,
                         "password": self.password,
-                        "secret": self.password}
+                        "secret": self.password,
+                        "banner_timeout": 200
+                        }
+        # autodetect_tries = 0
+        # while autodetect_tries < 10:
+        #     try:
+        #         guesser = SSHDetect(**autodetect_netmiko_args)
+        #         best_match = guesser.autodetect()
+        #         netmiko_args['device_type'] = best_match
+        #
+        #         dev = ConnectHandler(**netmiko_args)
+        #         dev.enable()
+        #
+        #
+        #
+        #     except NetmikoAuthenticationException:
+        #         # print(f"AUTHENTICATION ERROR FOR DEVICE: {device}")
+        #         return False
+        #     except:
+        #         autodetect_tries += 1
+        #         time.sleep(5)
+        #         tb = self.get_traceback()
+        #         self.log.info(f"DEVICE: {device} EXCEPTION02 THROWN IN discover_routes_ssh: {tb}")
 
-        try:
-            guesser = SSHDetect(**autodetect_netmiko_args)
-            best_match = guesser.autodetect()
-            netmiko_args['device_type'] = best_match
+            # raise Exception("Unable to determine OS or OS Not Supported.")
+        #     try:
+        #         if not netmiko_args['device_type']:
+        #             # Later put in unable to find OS
+        #             netmiko_args['device_type'] = "cisco_ios"
+        #         dev = ConnectHandler(**netmiko_args)
+        #         dev.enable()
+        #         CONN_SET = True
+        #     except Exception:
+        #         tb = self.get_traceback()
+        #         self.log.info(f"DEVICE: {device} EXCEPTION THROWN IN discover_routes_ssh: {tb}")
+        #
+        #         # print(tb)
+        #
+        #
+        # os = self._get_os_version(dev)
+        # netmiko_args['device_type'] = os
 
-        except NetmikoAuthenticationException:
-            # print(f"AUTHENTICATION ERROR FOR DEVICE: {device}")
-            return False
-        except:
-            try:
-                if not netmiko_args['device_type']:
-                    # Later put in unable to find OS
-                    netmiko_args['device_type'] = "cisco_ios"
-                dev = ConnectHandler(**netmiko_args)
-                dev.enable()
-            except Exception:
-                tb = self.get_traceback()
-                # print(tb)
-
-            CONN_SET = True
-            os = self._get_os_version(dev)
-            netmiko_args['device_type'] = os
-
-        if not CONN_SET:
-            dev = ConnectHandler(**netmiko_args)
+        # if not CONN_SET:
+        #     dev = ConnectHandler(**netmiko_args)
 
+        dev = ConnectHandler(**netmiko_args)
         dev.enable()
 
         routes = dev.send_command("show ip route vrf all", use_textfsm=True)
@@ -277,19 +316,116 @@ class dyagram:
                 i['routes'] = routes
                 break
 
-    def __discover_dynamic_routing_neighbors(self, device):
+    def get_device_type(self, device):
 
-        self.discover_ospf_neighbors(device)
+        autodetect_netmiko_args = {"device_type": "autodetect",
+                                   "host": device,
+                                   "username": self.username,
+                                   "password": self.password,
+                                   "banner_timeout": 200}
+        autodetect_tries = 0
+        while autodetect_tries < 10:
+            try:
 
-        self.discover_eigrp_neigbors(device)
+                guesser = SSHDetect(**autodetect_netmiko_args)
+                best_match = guesser.autodetect()
+                return best_match
 
-        self.discover_bgp_neighbors(device)
+
+            except NetmikoAuthenticationException:
+                # print(f"AUTHENTICATION ERROR FOR DEVICE: {device}")
+                return None
+
+            except:
+
+                if autodetect_tries > 10:
+                    tb = self.get_traceback()
+                    self.log.info(f"DEVICE: {device} EXCEPTION THROWN IN GET_DEVICE_TYPE: {tb}")
+                    raise Exception("Unable to access device")
+                autodetect_tries += 1
+                time.sleep(5)
+
+
+
+
+    def discover_dynamic_routing_neighbors(self, device):
+
+        try:
+
+            self.pbar.update(self.pbar_update_int)
+
+            self.discover_eigrp_neighbors(device)
+
+            self.discover_ospf_neighbors(device)
+
+            self.discover_bgp_neighbors(device)
+
+            self.pbar.update(self.pbar_update_int)
+        except:
+            tb = self.get_traceback()
+            self.log.info(f"DEVICE: {device} EXCEPTION THROWN IN discover_dynamic_routing_neighbors: {tb}")
 
     def discover_ospf_neighbors(self, device):
         pass
 
-    def discover_eigrp_neigbors(self, device):
-        pass
+    def discover_ospf_neighbors_ssh(self, device):
+        return None
+
+    def discover_ospf_neighbors_restconf(self):
+        return None
+
+    def discover_eigrp_neighbors(self, device):
+
+        try:
+            self.log.info(f"DEVICE: {device} - RESTCONF : Discovering EIGRP Neighbors - START")
+            discovered = self.discover_eigrp_neighbors_restconf()
+
+            if not discovered:
+                self.log.info(f"DEVICE: {device} - RESTCONF : Discovering EIGRP Neighbors - FAIL")
+                try:
+                    self.log.info(f"DEVICE: {device} - SSH : Discovering EIGRP Neighbors - START")
+                    self.discover_eigrp_neighbors_ssh(device)
+                    self.log.info(f"DEVICE: {device} - SSH : Discovering EIGRP Neighbors - SUCCESS")
+                except:
+                    tb = self.get_traceback()
+                    self.log.info(f"DEVICE: {device} EXCEPTION THROWN IN DISCOVERING EIGRP NEIGHBORS: {tb}")
+            else:
+                self.log.info(f"DEVICE: {device} - RESTCONF : Discovering EIGRP Neighbors - SUCCESS")
+        except:
+            pass
+
+
+    def discover_eigrp_neighbors_restconf(self):
+        return None
+
+    def discover_eigrp_neighbors_ssh(self, device):
+        os = self.get_device_type(device)
+        if os:
+            nm_args = self.netmiko_args_template.copy()
+            nm_args['host'] = device
+            nm_args['device_type'] = os
+
+        else:
+            raise Exception("Unable to determine OS.")
+
+
+        dev = ConnectHandler(**nm_args)
+        dev.enable()
+        if os in ['cisco_ios', 'cisco_nxos', 'cisco_xr']:
+            eigrp_output = dev.send_command("show ip eigrp neighbors vrf all") #textfsm not currently supported for this command
+            if isinstance(eigrp_output, str):
+                neighbor_ips = re.findall('\d+\.\d+\.\d+\.\d+', eigrp_output)
+                self.log.info(f"{device}: EIGRP NEIGHBOR IPs - {neighbor_ips}")
+            for i in self.topology['devices']:
+                if i['inventory_ip'] == device:
+                    i['dynamic_routing_neighbors']['eigrp'] = neighbor_ips
+                    break
+        else:
+            dev.disconnect()
+            raise Exception(f"OS {os} Not supported")
+        dev.disconnect()
+
+
 
     def discover_bgp_neighbors(self, device):
         pass
@@ -318,10 +454,10 @@ class dyagram:
                 return inventory_object
 
             except yaml.YAMLError as e:
-                print(e)
+                print(f"YAML ERROR: {e}")
 
     def _load_inventory(self):
-        self.pbar_update_int = 100 / len(self.inventory_object[self.site]) / 2 # 2 is the number of jobs each device hits CURRENTLY
+        self.pbar_update_int = 100 / len(self.inventory_object[self.site]) / 4 # CAN"T BE ODD SO GO UP ONE IF SO, 3 is the number of jobs each device hits CURRENTLY
         for ip in self.inventory_object[self.site]:
             self._devices_to_query.put(ip)
 
@@ -344,47 +480,40 @@ class dyagram:
         return True
 
     def _discover_lldp_neighbors_by_ssh(self, device):
+        self.log.info(f"DEVICE: {device} - SSH : STARTING MODULE: _discover_lldp_neighbors_by_ssh")
 
-        CONN_SET = False
+        os = self.get_device_type(device)
 
-        autodetect_netmiko_args = {"device_type": "autodetect",
-         "host": device,
-         "username": self.username,
-         "password": self.password}
-
-        netmiko_args = {"device_type": "",
+        netmiko_args = {"device_type": os,
                         "host": device,
                         "username": self.username,
                         "password": self.password,
-                        "secret": self.password}
-        try:
-            guesser = SSHDetect(**autodetect_netmiko_args)
-            best_match = guesser.autodetect()
-            netmiko_args['device_type'] = best_match
+                        "secret": self.password,
+                        "banner_timeout": 200}
 
-        except NetmikoAuthenticationException:
-           # print(f"AUTHENTICATION ERROR FOR DEVICE: {device}")
-            return False
-        except:
-            try:
-                if not netmiko_args['device_type']:
-                    # Later put in unable to find OS
-                    netmiko_args['device_type'] = "cisco_ios"
-                dev = ConnectHandler(**netmiko_args)
-                dev.enable()
-            except Exception:
-                tb = self.get_traceback()
-                #print(tb)
 
-            CONN_SET = True
-            os = self._get_os_version(dev)
-            netmiko_args['device_type'] = os
+            # try:
+            #     if not netmiko_args['device_type']:
+            #         # Later put in unable to find OS
+            #         netmiko_args['device_type'] = "cisco_ios"
+            #     dev = ConnectHandler(**netmiko_args)
+            #     dev.enable()
+            # except Exception:
+            #     tb = self.get_traceback()
+            #     self.log.info(f"DEVICE: {device} - SSH : ERROR DISCOVER LLDP NEIGHBORS - {tb}")
 
-        if not CONN_SET:
-            dev = ConnectHandler(**netmiko_args)
+
+            # CONN_SET = True
+            # os = self._get_os_version(dev)
+            # netmiko_args['device_type'] = os
+
+
+        dev = ConnectHandler(**netmiko_args)
+
+        self.log.info(f"DEVICE: {device} - SSH : SETTING ENABLE")
 
         dev.enable()
-
+        self.log.info(f"DEVICE: {device} - SSH : ENABLE SET")
 
         try:
             lldp_nei_json = self._get_lldp_neighbors_ssh_textfsm(dev)
@@ -392,25 +521,28 @@ class dyagram:
             if type(lldp_nei_json) == str:
                 raise ValueError("LLDP NEIGHBORS ARE STRs. Try without textfsm")
         except ValueError as e:
-            #print(e)
-            lldp_nei_json = self._get_lldp_neighbors_ssh_regex(dev)
+            try:
+                lldp_nei_json = self._get_lldp_neighbors_ssh_regex(dev)
+            except:
+                tb = self.get_traceback()
+                self.log.info(f"DEVICE: {device} EXCEPTION22 IN _discover_lldp_neighbors_by_ssh: {tb}")
         except Exception:
             tb = self.get_traceback()
-            #print(tb)
+            self.log.info(f"DEVICE: {device} - SSH : ERROR DISCOVER LLDP NEIGHBORS - {tb}")
 
         for i in self.topology["devices"]:
             if i['inventory_ip'] == device:
                 i['hostname'] = lldp_nei_json['hostname']
                 lldp_nei_json.pop("hostname")
-                i['layer2'].append(lldp_nei_json)
+                i['layer2'] = lldp_nei_json
                 break
 
         dev.disconnect()
         self._devices_queried.append(device)
 
-    def _create_netmiko_session(self):
+    def _create_netmiko_session(self, netmiko_kargs):
 
-        return ConnectHandler(**self.netmiko_args)
+        return ConnectHandler(**netmiko_kargs)
 
 
     def _get_chassis_ids(self, netmiko_session=None, os=None, restconf_session=None):
@@ -422,6 +554,7 @@ class dyagram:
                 resp = restconf_session.get(f"{restconf_session.base_url}/openconfig-interfaces:interfaces")
             except Exception:
                 tb = self.get_traceback()
+                self.log.info(f"DEVICE: {restconf_session.base_url} EXCEPTION01 THROWN IN _get_chassis_ids : {tb}")
                 #print(tb) LOG THIS
 
             if resp.status_code == 200:
@@ -494,7 +627,7 @@ class dyagram:
             for intf in oc_yang_resp.json()['interfaces']['interface']:
                 if 'neighbors' in intf.keys():
 
-                    neighbor_info = self._neighbor_template.copy()
+                    neighbor_info = self.lldp_neighbor_template.copy()
                     neighbor_info['hostname'] = intf['neighbors']['neighbor'][0]['state']['system-name']
                     neighbor_info['local_port'] = intf['name']
                     neighbor_info['neighbor_port'] = intf['neighbors']['neighbor'][0]['state']['port-id']
@@ -503,6 +636,8 @@ class dyagram:
 
         except Exception:
             tb = self.get_traceback()
+            self.log.info(f"DEVICE: {device} EXCEPTION01 THROWN IN _get_lldp_neighbors_restconf : {tb}")
+
             #print(tb)
 
         if oc_yang_resp.status_code == 200:
@@ -523,7 +658,7 @@ class dyagram:
 
 
         for neighbor in lldp_neighbors_output:
-            neighbor_info = self._neighbor_template.copy()
+            neighbor_info = self.lldp_neighbor_template.copy()
             neighbor_info['hostname'] = neighbor['neighbor']
             neighbor_info['local_port'] = neighbor['local_interface']
             neighbor_info['neighbor_port'] = neighbor['neighbor_interface']
@@ -641,11 +776,10 @@ class dyagram:
                     f"DEVICE: {device} - RESTCONF_OPENCONFIG_AND_NXOS_OS_DEVICE_YANG_CATCHALL : Querying hostname - FAILURE - TB: {tb}")
 
         else:
-            self.log.info(f"DEVICE: {netmiko_session['host']} - SSH : Querying for hostname - START")
+            self.log.info(f"DEVICE: {netmiko_session.host} - SSH : Querying for hostname - START")
             sh_run_output = netmiko_session.send_command("sh run | inc hostname")
-            #print(sh_run_output)
             hostname = re.search("hostname\s+(.*)", sh_run_output).group(1)
-            self.log.info(f"DEVICE: {netmiko_session['host']} - SSH : Querying for hostname - SUCCESSFUL")
+            self.log.info(f"DEVICE: {netmiko_session.host} - SSH : Querying for hostname - SUCCESSFUL")
 
             return hostname
         return None
@@ -693,40 +827,46 @@ class dyagram:
 def main():
 
     import argparse
+    try:
+        parser = argparse.ArgumentParser()
 
-    parser = argparse.ArgumentParser()
+        parser.add_argument('dyagram_args', nargs='*')
+        parser.add_argument("-v", action='store_true', dest='verbose')
 
-    parser.add_argument('dyagram_args', nargs='*')
-    parser.add_argument("-v", action='store_true', dest='verbose')
+        args = parser.parse_args()
 
-    args = parser.parse_args()
+        if args.dyagram_args[0].lower() == "init":
+            from dyagram.initialize import initialize
+            dyinit = initialize.dyagramInitialize()
+            dyinit.dy_init()
 
-    if args.dyagram_args[0].lower() == "init":
-        from dyagram.initialize import initialize
-        dyinit = initialize.dyagramInitialize()
-        dyinit.dy_init()
+        if args.dyagram_args[0].lower() == "discover":
+            try:
+                dy = dyagram(verbose=args.verbose)
+                dy.discover()
+            except:
+                tb = dyagram.get_traceback()
+                print(f"DEVICE: MAIN EXCEPTION : {tb}")
 
-    if args.dyagram_args[0].lower() == "discover":
-
-        dy = dyagram(verbose=args.verbose)
-        dy.discover()
-
-    if args.dyagram_args[0].lower() == "site":
-        from dyagram.sites.sites import sites
-        s = sites()
-        if len(args.dyagram_args) == 1:
-            s.list_sites_in_cli()
-        elif len(args.dyagram_args) > 1:
-            if args.dyagram_args[1].lower() == "new":
-                try:
-                    s.make_new_site(args.dyagram_args[2])
-                except Exception as e:
-                    print("Missing argument : <site>")
-            if args.dyagram_args[1].lower() == "switch":
-                try:
-                    s.switch_site(args.dyagram_args[2].lower())
-                except:
-                    print(f"Site {args.dyagram_args[2]} doesn't exist")
+        if args.dyagram_args[0].lower() == "site":
+            from dyagram.sites.sites import sites
+            s = sites()
+            if len(args.dyagram_args) == 1:
+                s.list_sites_in_cli()
+            elif len(args.dyagram_args) > 1:
+                if args.dyagram_args[1].lower() == "new":
+                    try:
+                        s.make_new_site(args.dyagram_args[2])
+                    except Exception as e:
+                        print("Missing argument : <site>")
+                if args.dyagram_args[1].lower() == "switch":
+                    try:
+                        s.switch_site(args.dyagram_args[2].lower())
+                    except:
+                        print(f"Site {args.dyagram_args[2]} doesn't exist")
+    except:
+        tb = dyagram.get_traceback()
+        print(f"DEVICE: MAIN EXCEPTION : {tb}")
 
 
 if __name__ == "__main__":
